#!/usr/bin/python3

import re


MULTIPLES = {
    'B': 1,
    'kB': 1000,
    'KiB': 1024,
    'MB': 1000 ** 2,
    'MiB': 1024 ** 2,
    'GB': 1000 ** 3,
    'GiB': 1024 ** 3,
    'TB': 1000 ** 4,
    'TiB': 1024 ** 4,
}

re_size_with_unit = re.compile(
    r'^(?P<value>[\d\.]+)(?P<unit>k?B|KiB|[MGT]i?B)$')
re_size_without_unit = re.compile(r'^(?P<value>[\d\.]+)$')


class SizeValue(object):
    def __init__(self, value, unit=None):
        if not unit:
            value, unit = self.parse_value(value)

        if unit not in MULTIPLES:
            raise Exception(f"Bad unit {unit}")

        multiply = MULTIPLES.get(unit)
        self._value = int(float(value) * multiply)

    def __str__(self):
        return f"{self._value}B"

    @staticmethod
    def parse_value(value):
        value = value.strip()
        match = re_size_with_unit.match(value)
        if match:
            _value = match.group('value')
            _unit = match.group('unit') or 'B'
            return _value, _unit

        match = re_size_without_unit.match(value)
        if match:
            _value = match.group('value')
            return _value, 'B'

        raise Exception(f"Bad value {value}")

    def convert(self, units=None, show_units=True):
        if not units:
            units = 'B'

        if units not in MULTIPLES:
            raise Exception(f"Bad target unit {units}")

        divider = MULTIPLES.get(units)
        value = float(self._value / divider)
        if show_units:
            return f"{value}{units}"

        return f"{value}"


def convert_units(value, from_units=None, to_units='B', show_units=True):
    size = SizeValue(value, from_units)
    return size.convert(to_units, show_units)


class FilterModule(object):
    def filters(self):
        return {
            'convert_units': convert_units,
        }
